@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2017 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2017-2018 -- leonerd@leonerd.org.uk
 
 package Device::Chip::MAX11200;
 
@@ -9,9 +9,10 @@ use strict;
 use warnings;
 use base qw( Device::Chip );
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use Carp;
+use Future::AsyncAwait;
 use Data::Bitfield 0.02 qw( bitfield boolfield enumfield );
 use List::Util qw( first );
 
@@ -70,19 +71,18 @@ use constant {
    REG_SCGC  => 8,
 };
 
-sub read_register
+async sub read_register
 {
    my $self = shift;
    my ( $reg, $len ) = @_;
 
    $len //= 1;
 
-   $self->protocol->readwrite(
+   my $bytes = await $self->protocol->readwrite(
       pack "C a*", 0xC1 | ( $reg << 1 ), "\0" x $len
-   )->then( sub {
-      my ( $bytes ) = @_;
-      Future->done( substr $bytes, 1 );
-   });
+   );
+
+   return substr $bytes, 1;
 }
 
 sub write_register
@@ -136,14 +136,13 @@ bitfield { format => "bytes-LE" }, STAT =>
    RATE  => enumfield(4, @RATES),
    SYSOR => boolfield(7);
 
-sub read_status
+async sub read_status
 {
    my $self = shift;
 
-   $self->read_register( REG_STAT )->then( sub {
-      my ( $bytes ) = @_;
-      return Future->done( unpack_STAT( $bytes ) );
-   });
+   my $bytes = await $self->read_register( REG_STAT );
+
+   return unpack_STAT( $bytes );
 }
 
 =head2 read_config
@@ -184,17 +183,15 @@ bitfield { format => "bytes-LE" }, CONFIG =>
    NOSYSG => boolfield(8+4),
    DGAIN  => enumfield(8+5, qw( 1 2 4 8 16 ));
 
-sub read_config
+async sub read_config
 {
    my $self = shift;
 
-   Future->needs_all(
-      $self->read_register( REG_CTRL1 ),
-      $self->read_register( REG_CTRL3 ),
-   )->then( sub {
-      my ( $ctrl1, $ctrl3 ) = @_;
-      Future->done( $self->{config} = { unpack_CONFIG( $ctrl1 . $ctrl3 ) } );
-   });
+   my ( $ctrl1, $ctrl3 ) = await Future->needs_all(
+      $self->read_register( REG_CTRL1 ), $self->read_register( REG_CTRL3 )
+   );
+
+   return $self->{config} = { unpack_CONFIG( $ctrl1 . $ctrl3 ) };
 }
 
 =head2 change_config
@@ -206,23 +203,20 @@ their existing values.
 
 =cut
 
-sub change_config
+async sub change_config
 {
    my $self = shift;
    my %changes = @_;
 
-   ( $self->{config} ? Future->done( $self->{config} ) : $self->read_config )
-   ->then( sub {
-      my ( $config ) = @_;
+   my $config = $self->{config} // await $self->read_config;
 
-      $self->{config} = { %$config, %changes };
-      my $ctrlb = pack_CONFIG( %{ $self->{config} } );
+   $self->{config} = { %$config, %changes };
+   my $ctrlb = pack_CONFIG( %{ $self->{config} } );
 
-      Future->needs_all(
-         $self->write_register( REG_CTRL1, substr $ctrlb, 0, 1 ),
-         $self->write_register( REG_CTRL3, substr $ctrlb, 1, 1 ),
-      );
-   });
+   await Future->needs_all(
+      $self->write_register( REG_CTRL1, substr $ctrlb, 0, 1 ),
+      $self->write_register( REG_CTRL3, substr $ctrlb, 1, 1 ),
+   );
 }
 
 =head2 selfcal
@@ -307,14 +301,13 @@ either signed or unsigned as per the C<FORMAT> configuration.
 
 =cut
 
-sub read_adc
+async sub read_adc
 {
    my $self = shift;
 
-   $self->read_register( REG_DATA, 3 )->then( sub {
-      my ( $bytes ) = @_;
-      return Future->done( unpack "L>", "\0$bytes" );
-   });
+   my $bytes = await $self->read_register( REG_DATA, 3 );
+
+   return unpack "L>", "\0$bytes";
 }
 
 =head2 write_gpios
@@ -339,14 +332,13 @@ sub write_gpios
    $self->write_register( REG_CTRL2, pack "C", ( $dir << 4 ) | $values );
 }
 
-sub read_gpios
+async sub read_gpios
 {
    my $self = shift;
 
-   $self->read_register( REG_CTRL2 )->then( sub {
-      my ( $bytes ) = @_;
-      return Future->done( 0x0F & unpack "C", $bytes );
-   });
+   my $bytes = await $self->read_register( REG_CTRL2 );
+
+   return 0x0F & unpack "C", $bytes;
 }
 
 =head2 Calibration Registers
@@ -376,11 +368,9 @@ foreach (
 
    no strict 'refs';
 
-   *{"read_$name"} = sub {
-      $_[0]->read_register( $reg, 3 )->then( sub {
-         my ( $bytes ) = @_;
-         Future->done( unpack "I>", "\0" . $bytes );
-      });
+   *{"read_$name"} = async sub {
+      my $bytes = await $_[0]->read_register( $reg, 3 );
+      return unpack "I>", "\0" . $bytes;
    };
 
    *{"write_$name"} = sub {
